package com.sgrsoft.vpe_aos_demo
import android.content.Context
import android.content.res.Configuration
import android.os.Bundle
import android.util.Log
import android.view.MotionEvent
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.webkit.WebView
import androidx.appcompat.app.AppCompatActivity
import com.navercloud.vpe.sdk.*
import com.sgrsoft.vpe_aos_demo.databinding.ActivityMainBinding
import org.json.JSONObject
import android.util.DisplayMetrics
import android.widget.FrameLayout
import okhttp3.*
import java.io.IOException
import android.media.MediaDrm
import android.os.Build
import java.util.*

class MainActivity : AppCompatActivity() {

    private lateinit var binding: ActivityMainBinding
    private lateinit var webView: WebView
    private val packageId = BuildConfig.APPLICATION_ID;
    private var player = NCPlayerView()
    private val client = OkHttpClient()
    private var isFullscreen = false
    private var isInPipMode = false
    private val webContentsDebuggingEnabled = false //웹 디버거 허용 여부


    /**
     * Video Player Enhancement (VPE) 라이선스키
     */
    private val licenseKey = "a624346b30f93b892c4fa2f636a57c83"; //라이선스키

    /**
     * 플레이어 비율
     */
    private var aspectRatio = 16.0f / 9.0f //16/9로 조절

    /**
     * 화면 캡쳐 금지 여부 설정
      - 안드로이드 DRM은 화면캡쳐를 방지하지 않습니다.
      - isScreenCaptureDeny = true 시 캡쳐 및 녹화 방지가 VPE 플레이어에서 적용됩니다.
     */
    private val isScreenCaptureDeny = true


    /**
     * onCreate
     */
    override fun onCreate(savedInstanceState: Bundle?) {

        super.onCreate(savedInstanceState)

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        webView = findViewById(R.id.webView)
        WebView.setWebContentsDebuggingEnabled(webContentsDebuggingEnabled)

        val delegate = PlayerDelegate()
        NCPlayer.delegate = delegate
        player.webView = webView

        // 기기가 Widevine DRM을 지원하는지 확인
        val widevineUUID = UUID.fromString("edef8ba9-79d6-4ace-a3c8-27dcd51d21ed")

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
            val isWidevineSupported = MediaDrm.isCryptoSchemeSupported(widevineUUID)

            if (isWidevineSupported) {
                Log.d("ncplayer DRM Check", "Widevine is supported on this device.")
            } else {
                Log.e("ncplayer DRM Check", "Widevine is NOT supported on this device.")
            }
        } else {
            Log.e("ncplayer DRM Check", "Widevine DRM is not supported on devices below Android 4.3 (API level 18).")
        }

        startPlayer();

    }

    private fun startPlayer() {

        // 서버에서 플레이어 설정 받아오기 예제 -----------------------
        /**
         * - TIP : Backend 서버에서 설정을 받아 처리하는 예제 (fetchPlayerOptionsFromServer)
         * - ※ DRM 콘텐츠 이용시 Backend 서버에서 설정을 받아 처리하세요.
         *
         * Backend 서버 예제 목업 API 목록
         * - 일반 hls 예제 : https://vpe.sgrsoft.com/api/playUrl
         * - Pallycon DRM 테스트 예제 : https://vpe.sgrsoft.com/api/drmTestPallycon
         * - One Click Multi DRM 테스트 예제 : https://vpe.sgrsoft.com/api/drmTest
         */

        fetchPlayerOptionsFromServer("https://vpe.sgrsoft.com/api/playUrl")


        // 코드에서 직접 설정하는 예제  ---------------------------
        /**
         * Tip : 플레이어에 설정을 전달하는 예제 (setupPlayer)
         * - 서버방식을 이용할때는 아래 코드를 제거하세요.
         */

        /*setupPlayer("""
            {
                 "playlist":[
                    {
                        "file":"https://m4qgahqg2249.edge.naverncp.com/hls/a4oif2oPHP-HlGGWOFm29A__/endpoint/sample/221027_NAVER_Cloud_intro_Long_ver_AVC_,FHD_2Pass_30fps,HD_2Pass_30fps,SD_2Pass_30fps,.mp4.smil/master.m3u8",
                        "poster":"https://2ardrvaj2252.edge.naverncp.com/endpoint/sample/221027_NAVER_Cloud_intro_Long_ver_01.jpg",
                        "description":{
                            "title":"네이버클라우드 테스트 영상",
                            "created_at":"2024.07.13",
                            "profile_name":"네이버클라우드",
                            "profile_image":"https://nnbkegvqsbcu5297614.cdn.ntruss.com/profile/202208/d127c8db642716d84b3201f1d152e52a.png"
                        }
                    }

                 ],
                 "autostart":true,
                 "muted":true,
                 "progressBarColor":"#ff0000"
            }

        """);
        */

    }

    /**
     * Backend 서버에서 플레이어 설정을 통신
     */
    private fun fetchPlayerOptionsFromServer(url: String) {
        val request = Request.Builder().url(url).build()

        client.newCall(request).enqueue(object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                Log.e("MainActivity", "Failed to fetch player options", e)
            }

            override fun onResponse(call: Call, response: Response) {
                response.body?.string()?.let { json ->
                    runOnUiThread {
                        setupPlayer(json,"json")
                    }
                }
            }
        })
    }

    /**
     * 플레이어에 설정을 전달 및 플레이어 생성
     */
    private fun setupPlayer(playerOptions: String,resultType:String = "") {

        NCPlayer.prefs.setString("appId", packageId)
        NCPlayer.prefs.setString("licenseKey", licenseKey)
        NCPlayer.prefs.setString("isScreenCaptureDeny", isScreenCaptureDeny.toString())

        if (resultType == "json") {
            try {
                val jsonObject = JSONObject(playerOptions)
                var playerOptionsServer = jsonObject.toString()

                //플레이어 생성
                player.create(playerOptionsServer, this)


            } catch (e: Exception) {
                Log.e("ncplayer MainActivity", "Invalid JSON format", e)
            }
        }else{
            //플레이어 생성
            player.create(playerOptions, this)

        }


        //플레이어 비율 조정
        webViewAspectRatio(aspectRatio)

        //플레이어 이벤트 바인딩
        playerEventBind()

    }

    /**
     * 플레이어 기능 버튼에 이벤트 바인딩
     */
    private fun playerEventBind(){

        //구간이동
        binding.submit.setOnClickListener {
            val currentTime = binding.currentTimeText.text.toString()
            player.currentTime(currentTime)
        }

        //재생
        binding.play.setOnClickListener {
            player.videoPlay()
        }

        //일시정지
        binding.pause.setOnClickListener {
            player.pause()
        }

        //음소거
        binding.mute.setOnClickListener {
            player.mute()
        }

        //컨트롤바 활성화
        binding.controlBarActive.setOnClickListener {
            player.controlBarActive()
        }

        //컨트롤바 비활성화
        binding.controlBarDeactive.setOnClickListener {
            player.controlBarDeactive()
        }

        //플레이어 제거
        binding.destroy.setOnClickListener {
            player.destroy()
        }

    }


    /**
     * Picture-in-Picture mode enter callback
     * 앱이 백그라운드에 있을때 PIP 모드로 전환 호출
     */
    override fun onUserLeaveHint() {
        player.enterPip(this, 16, 9) //pip 실행시 비율 설정
        isInPipMode = true
    }

    /**
     * Picture-in-Picture mode change callback
     * PIP 상대 변경 감지
     */
    override fun onPictureInPictureModeChanged(isInPictureInPictureMode: Boolean, newConfig: Configuration?) {
        super.onPictureInPictureModeChanged(isInPictureInPictureMode, newConfig)
        isInPipMode = isInPictureInPictureMode

        if (isInPictureInPictureMode) {
            hideUiElements()
        } else {
            showUiElements()
            player.leavePip()
        }
    }

    /**
     * 화면 터치시 키보드 숨기기
     */
    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        if (currentFocus != null) {
            val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
            imm.hideSoftInputFromWindow(currentFocus!!.windowToken, 0)
        }
        return super.dispatchTouchEvent(ev)
    }




    /**
     * 화면 회전시 플레이어 상태 변경
     */
    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)

        if (newConfig.orientation == Configuration.ORIENTATION_PORTRAIT) {

            if(!isInPipMode) {
                NCPlayer.delegate?.NCPlayerIsLandscape(false)
                isFullscreen = false
                player.fullscreenOff(this, true)
            }

        } else if (newConfig.orientation == Configuration.ORIENTATION_LANDSCAPE) {

            if(!isInPipMode) {
                NCPlayer.delegate?.NCPlayerIsLandscape(true)
                isFullscreen = true
                player.fullscreenOn(this, true)
            }
        }

        webViewAspectRatio(aspectRatio)
    }


    /**
     * 화면 비율 조절 함수
     */
    private fun webViewAspectRatio(aspectRatio: Float) {
        val displayMetrics = DisplayMetrics()
        windowManager.defaultDisplay.getMetrics(displayMetrics)
        val width = displayMetrics.widthPixels
        val height = displayMetrics.heightPixels

        if(width < height){
            val adjustedHeight = (width / aspectRatio).toInt()
            val layoutParams = FrameLayout.LayoutParams(width, adjustedHeight)
            webView.layoutParams = layoutParams
        }else{
            if(isInPipMode || isFullscreen){ //pip , fullscreen 상태일때 꽉차게
                webView.layoutParams = FrameLayout.LayoutParams(width, height)
            }else {
                //이 부분은 앱 컨샙에 따라 조절
                val adjustedHeight = height / 2
                webView.layoutParams = FrameLayout.LayoutParams(width, adjustedHeight)
            }
        }

    }


    /**
     * PIP 전환시 UI 변경
     */
    private fun hideUiElements() {
        binding.submit.visibility = View.GONE
        binding.play.visibility = View.GONE
        binding.pause.visibility = View.GONE
        binding.mute.visibility = View.GONE
        binding.controlBarActive.visibility = View.GONE
        binding.controlBarDeactive.visibility = View.GONE
        binding.destroy.visibility = View.GONE
        binding.currentTimeText.visibility = View.GONE
    }

    /**
     * PIP 해제시 UI 변경
     */
    private fun showUiElements() {
        binding.submit.visibility = View.VISIBLE
        binding.play.visibility = View.VISIBLE
        binding.pause.visibility = View.VISIBLE
        binding.mute.visibility = View.VISIBLE
        binding.controlBarActive.visibility = View.VISIBLE
        binding.controlBarDeactive.visibility = View.VISIBLE
        binding.destroy.visibility = View.VISIBLE
        binding.currentTimeText.visibility = View.VISIBLE
    }
}



/**
 * NCPlayerDelegate
 * 플레이어 발생 이벤트 리스너
 */
class PlayerDelegate : NCPlayerEventInterface  {
    private val TAG = "PlayerDelegate"

    //컨트롤바 활성화시 이벤트
    override fun NCPlayerOnControlbarActive(active: JSONObject?) {
        Log.d(TAG, "NCPlayerOnControlbarActive: $active")
    }

    //동영상의 재생이 완료되었을때 이벤트
    override fun NCPlayerOnEnd() {
        Log.d(TAG, "NCPlayerOnEnd: ")
    }

    //이전 동영상 이동 이벤트
    override fun NCPlayerOnPrevTrack(data: JSONObject?) {
        Log.d(TAG, "NCPlayerOnPrevTrack: $data")
    }
    //다음동영상 넘어갈때 이벤트
    override fun NCPlayerOnNextTrack(data: JSONObject?) {
        Log.d(TAG, "NCPlayerOnNextTrack: $data")
    }

    //일시정지 이벤트
    override fun NCPlayerOnPause() {
        Log.d(TAG, "NCPlayerOnPause: ")
    }

    //재생 이벤트
    override fun NCPlayerOnPlay() {
        Log.d(TAG, "NCPlayerOnPlay: ")
    }

    //플레이중 시간이 변경되었을때 이벤트
    override fun NCPlayerOnTimeUpdate(data: JSONObject?) {
        Log.d(TAG, "NCPlayerOnTimeUpdate: $data")
        Log.d(TAG, "NCPlayerOnTimeUpdate: ${data?.get("currentTime")}")
    }

    //풀스크린 상태 변경 감지 이벤트
    override fun NCPlayerOnFullscreen(state: Boolean) {
        Log.d(TAG, "NCPlayerOnFullscreen: $state")
    }

    //풀스크린 해제 이벤트
    override fun NCPlayerOnFullscreenOff() {
        Log.d(TAG, "NCPlayerOnFullscreenOff: off")

    }

    //풀스크린 켜짐 이벤트
    override fun NCPlayerOnFullscreenOn() {
        Log.d(TAG, "NCPlayerOnFullscreenOn: on")
    }

    //디바이스 가로 회전시 이벤트
    override fun NCPlayerIsLandscape(state: Boolean) {
        Log.d(TAG, "NCPlayerIsLandscape: $state")
    }

    //네트워크 상태 변경 이벤트
    override fun NCPlayerNetworkStatus(state: String) {
        Log.d(TAG, "NCPlayerNetworkStatus: $state")
    }
}